#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2025/9/11 13:00
Desc: 东方财富-港股-公司概况
https://emweb.securities.eastmoney.com/PC_HKF10/pages/home/index.html?code=03900&type=web&color=w#/CompanyProfile
"""

import pandas as pd
import requests


def stock_hk_security_profile_em(symbol: str = "03900") -> pd.DataFrame:
    """
    东方财富-港股-证券资料
    https://emweb.securities.eastmoney.com/PC_HKF10/pages/home/index.html?code=03900&type=web&color=w#/CompanyProfile
    :param symbol: 股票代码
    :type symbol: str
    :return: 证券资料
    :rtype: pandas.DataFrame
    """
    url = "https://datacenter.eastmoney.com/securities/api/data/v1/get"
    params = {
        "reportName": "RPT_HKF10_INFO_SECURITYINFO",
        "columns": "SECUCODE,SECURITY_CODE,SECURITY_NAME_ABBR,SECURITY_TYPE,LISTING_DATE,ISIN_CODE,BOARD,"
        "TRADE_UNIT,TRADE_MARKET,GANGGUTONGBIAODISHEN,GANGGUTONGBIAODIHU,PAR_VALUE,"
        "ISSUE_PRICE,ISSUE_NUM,YEAR_SETTLE_DAY",
        "quoteColumns": "",
        "filter": f'(SECUCODE="{symbol}.HK")',
        "pageNumber": "1",
        "pageSize": "200",
        "sortTypes": "",
        "sortColumns": "",
        "source": "F10",
        "client": "PC",
        "v": "04748497219912483",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["result"]["data"])
    field_mapping = {
        "BOARD": "板块",
        "GANGGUTONGBIAODIHU": "是否沪港通标的",
        "GANGGUTONGBIAODISHEN": "是否深港通标的",
        "ISIN_CODE": "ISIN（国际证券识别编码）",
        "ISSUE_NUM": "发行量(股)",
        "ISSUE_PRICE": "发行价",
        "LISTING_DATE": "上市日期",
        "PAR_VALUE": "每股面值",
        "SECUCODE": "证券代码",
        "SECURITY_NAME_ABBR": "证券简称",
        "SECURITY_TYPE": "证券类型",
        "TRADE_MARKET": "交易所",
        "TRADE_UNIT": "每手股数",
        "YEAR_SETTLE_DAY": "年结日",
    }
    temp_df.rename(columns=field_mapping, inplace=True)
    temp_df = temp_df[
        [
            "证券代码",
            "证券简称",
            "上市日期",
            "证券类型",
            "发行价",
            "发行量(股)",
            "每手股数",
            "每股面值",
            "交易所",
            "板块",
            "年结日",
            "ISIN（国际证券识别编码）",
            "是否沪港通标的",
            "是否深港通标的",
        ]
    ]
    return temp_df


def stock_hk_company_profile_em(symbol: str = "03900") -> pd.DataFrame:
    """
    东方财富-港股-公司资料
    https://emweb.securities.eastmoney.com/PC_HKF10/pages/home/index.html?code=03900&type=web&color=w#/CompanyProfile
    :param symbol: 股票代码
    :type symbol: str
    :return: 公司资料
    :rtype: pandas.DataFrame
    """
    url = "https://datacenter.eastmoney.com/securities/api/data/v1/get"
    params = {
        "reportName": "RPT_HKF10_INFO_ORGPROFILE",
        "columns": "SECUCODE,SECURITY_CODE,ORG_NAME,ORG_EN_ABBR,BELONG_INDUSTRY,FOUND_DATE,CHAIRMAN,"
        "SECRETARY,ACCOUNT_FIRM,REG_ADDRESS,ADDRESS,YEAR_SETTLE_DAY,EMP_NUM,ORG_TEL,ORG_FAX,ORG_EMAIL,"
        "ORG_WEB,ORG_PROFILE,REG_PLACE",
        "quoteColumns": "",
        "filter": f'(SECUCODE="{symbol}.HK")',
        "pageNumber": "1",
        "pageSize": "200",
        "sortTypes": "",
        "sortColumns": "",
        "source": "F10",
        "client": "PC",
        "v": "04748497219912483",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["result"]["data"])
    field_mapping = {
        "ACCOUNT_FIRM": "核数师",
        "ADDRESS": "办公地址",
        "BELONG_INDUSTRY": "所属行业",
        "CHAIRMAN": "董事长",
        "EMP_NUM": "员工人数",
        "FOUND_DATE": "公司成立日期",
        "ORG_EMAIL": "E-MAIL",
        "ORG_EN_ABBR": "英文名称",
        "ORG_FAX": "传真",
        "ORG_NAME": "公司名称",
        "ORG_PROFILE": "公司介绍",
        "ORG_TEL": "联系电话",
        "ORG_WEB": "公司网址",
        "REG_ADDRESS": "注册地址",
        "REG_PLACE": "注册地",
        "SECRETARY": "公司秘书",
        "SECUCODE": "股票代码",
        "SECURITY_CODE": "证券代码",
        "YEAR_SETTLE_DAY": "年结日",
    }
    temp_df.rename(columns=field_mapping, inplace=True)
    temp_df = temp_df[
        [
            "公司名称",
            "英文名称",
            "注册地",
            "注册地址",
            "公司成立日期",
            "所属行业",
            "董事长",
            "公司秘书",
            "员工人数",
            "办公地址",
            "公司网址",
            "E-MAIL",
            "年结日",
            "联系电话",
            "核数师",
            "传真",
            "公司介绍",
        ]
    ]
    return temp_df


def stock_hk_financial_indicator_em(symbol: str = "03900") -> pd.DataFrame:
    """
    东方财富-港股-核心必读-最新指标
    https://emweb.securities.eastmoney.com/PC_HKF10/pages/home/index.html?code=03900&type=web&color=w#/CoreReading
    :param symbol: 股票代码
    :type symbol: str
    :return: 财务指标
    :rtype: pandas.DataFrame
    """
    url = "https://datacenter.eastmoney.com/securities/api/data/v1/get"
    params = {
        "reportName": "RPT_CUSTOM_HKF10_FN_MAININDICATORMAX",
        "columns": "ORG_CODE,SECUCODE,SECURITY_CODE,SECURITY_NAME_ABBR,SECURITY_INNER_CODE,REPORT_DATE,BASIC_EPS,"
        "PER_NETCASH_OPERATE,BPS,BPS_NEDILUTED,COMMON_ACS,PER_SHARES,ISSUED_COMMON_SHARES,HK_COMMON_SHARES,"
        "TOTAL_MARKET_CAP,HKSK_MARKET_CAP,OPERATE_INCOME,OPERATE_INCOME_SQ,OPERATE_INCOME_QOQ,"
        "OPERATE_INCOME_QOQ_SQ,HOLDER_PROFIT,HOLDER_PROFIT_SQ,HOLDER_PROFIT_QOQ,HOLDER_PROFIT_QOQ_SQ,PE_TTM,"
        "PE_TTM_SQ,PB_TTM,PB_TTM_SQ,NET_PROFIT_RATIO,NET_PROFIT_RATIO_SQ,ROE_AVG,ROE_AVG_SQ,ROA,"
        "ROA_SQ,DIVIDEND_TTM,DIVIDEND_LFY,DIVI_RATIO,DIVIDEND_RATE,IS_CNY_CODE",
        "quoteColumns": "",
        "filter": f'(SECUCODE="{symbol}.HK")',
        "pageNumber": "1",
        "pageSize": "200",
        "sortTypes": "-1",
        "sortColumns": "REPORT_DATE",
        "source": "F10",
        "client": "PC",
        "v": "07945646099062258",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["result"]["data"])
    field_mapping = {
        "SECURITY_CODE": "股票代码",
        "BASIC_EPS": "基本每股收益(元)",
        "BPS": "每股净资产(元)",
        "COMMON_ACS": "法定股本(股)",
        "PER_SHARES": "每手股",
        "DIVIDEND_TTM": "每股股息TTM(港元)",
        "DIVI_RATIO": "派息比率(%)",
        "ISSUED_COMMON_SHARES": "已发行股本(股)",
        "HK_COMMON_SHARES": "已发行股本-H股(股)",
        "PER_NETCASH_OPERATE": "每股经营现金流(元)",
        "DIVIDEND_RATE": "股息率TTM(%)",
        "TOTAL_MARKET_CAP": "总市值(港元)",
        "HKSK_MARKET_CAP": "港股市值(港元)",
        "OPERATE_INCOME": "营业总收入",
        "OPERATE_INCOME_QOQ": "营业总收入滚动环比增长(%)",
        "NET_PROFIT_RATIO": "销售净利率(%)",
        "HOLDER_PROFIT": "净利润",
        "HOLDER_PROFIT_QOQ": "净利润滚动环比增长(%)",
        "ROE_AVG": "股东权益回报率(%)",
        "PE_TTM": "市盈率",
        "PB_TTM": "市净率",
        "ROA": "总资产回报率(%)",
    }
    temp_df.rename(columns=field_mapping, inplace=True)
    temp_df = temp_df[
        [
            "基本每股收益(元)",
            "每股净资产(元)",
            "法定股本(股)",
            "每手股",
            "每股股息TTM(港元)",
            "派息比率(%)",
            "已发行股本(股)",
            "已发行股本-H股(股)",
            "每股经营现金流(元)",
            "股息率TTM(%)",
            "总市值(港元)",
            "港股市值(港元)",
            "营业总收入",
            "营业总收入滚动环比增长(%)",
            "销售净利率(%)",
            "净利润",
            "净利润滚动环比增长(%)",
            "股东权益回报率(%)",
            "市盈率",
            "市净率",
            "总资产回报率(%)",
        ]
    ]
    return temp_df


def stock_hk_dividend_payout_em(symbol: str = "03900") -> pd.DataFrame:
    """
    东方财富-港股-核心必读-分红派息
    https://emweb.securities.eastmoney.com/PC_HKF10/pages/home/index.html?code=03900&type=web&color=w#/CoreReading
    :param symbol: 股票代码
    :type symbol: str
    :return: 分红派息
    :rtype: pandas.DataFrame
    """
    url = "https://datacenter.eastmoney.com/securities/api/data/v1/get"
    params = {
        "reportName": "RPT_HKF10_MAIN_DIVBASIC",
        "columns": "SECURITY_CODE,UPDATE_DATE,REPORT_TYPE,EX_DIVIDEND_DATE,DIVIDEND_DATE,"
        "TRANSFER_END_DATE,YEAR,PLAN_EXPLAIN,IS_BFP",
        "quoteColumns": "",
        "filter": f'(SECURITY_CODE="{symbol}")(IS_BFP="0")',
        "pageNumber": "1",
        "pageSize": "200",
        "sortTypes": "-1,-1",
        "sortColumns": "NOTICE_DATE,EX_DIVIDEND_DATE",
        "source": "F10",
        "client": "PC",
        "v": "035584639294227527",
    }
    r = requests.get(url, params=params)
    data_json = r.json()

    field_mapping = {
        "SECURITY_CODE": "股票代码",
        "UPDATE_DATE": "最新公告日期",
        "REPORT_TYPE": "分配类型",
        "EX_DIVIDEND_DATE": "除净日",
        "DIVIDEND_DATE": "发放日",
        "TRANSFER_END_DATE": "截至过户日",
        "YEAR": "财政年度",
        "PLAN_EXPLAIN": "分红方案",
        "IS_BFP": "IS_BFP",
    }
    columns = [
        "最新公告日期",
        "财政年度",
        "分红方案",
        "分配类型",
        "除净日",
        "截至过户日",
        "发放日",
    ]

    temp_df = pd.DataFrame(columns=columns)
    if data_json["result"] is not None:
        temp_df = pd.DataFrame(data_json["result"]["data"])
        temp_df.rename(columns=field_mapping, inplace=True)
        temp_df = temp_df[columns]
        temp_df["最新公告日期"] = pd.to_datetime(
            temp_df["最新公告日期"], errors="coerce"
        ).dt.date
        temp_df["除净日"] = pd.to_datetime(temp_df["除净日"], errors="coerce").dt.date
        temp_df["发放日"] = pd.to_datetime(
            temp_df["发放日"], format="%Y/%m/%d", errors="coerce"
        ).dt.date
    return temp_df


if __name__ == "__main__":
    stock_hk_security_profile_em_df = stock_hk_security_profile_em(symbol="03900")
    print(stock_hk_security_profile_em_df)

    stock_hk_company_profile_em_df = stock_hk_company_profile_em(symbol="03900")
    print(stock_hk_company_profile_em_df)

    stock_hk_financial_indicator_em_df = stock_hk_financial_indicator_em(symbol="03900")
    print(stock_hk_financial_indicator_em_df)

    stock_hk_dividend_payout_em_df = stock_hk_dividend_payout_em(symbol="03900")
    print(stock_hk_dividend_payout_em_df)



# -*- coding:utf-8 -*-
# !/usr/bin/env python
"""
Date: 2026/2/22 13:00
Desc: 天天基金网-基金数据-分红送配
https://fund.eastmoney.com/data/fundfenhong.html
"""

import pandas as pd
import requests

from akshare.utils.tqdm import get_tqdm


def fund_fh_em(
    year: str = "2025",
    typ: str = "",
    rank: str = "BZDM",
    sort: str = "asc",
    page: int = -1,
) -> pd.DataFrame:
    """
    天天基金网-基金数据-分红送配-基金分红
    https://fund.eastmoney.com/data/fundfenhong.html#DJR,desc,1,,,
    :param year: 查询年份
    :type year: str
    :param typ: 基金类型；空串表示全部; choice of {"指数型-其他", "指数型-海外股票", "指数型-固收", "指数型-股票", "债券型-中短债",
    "债券型-长债", "债券型-理财", "债券型-混合债", "债券型-混合一级", "债券型-混合二级", "货币型-普通货币", "货币型-浮动净值",
    "混合型-平衡", "混合型-偏债", "混合型-偏股", "混合型-灵活", "混合型-绝对收益", "股票型", "REITs", "Reits", "QDII-商品",
    "QDII-普通股票", "QDII-混合债", "QDII-混合偏股", "QDII-纯债", "QDII-REITs", "FOF"}
    :type typ: str
    :param rank: 排序字段；choice of {"BZDM", "ABBNAME", "DJR", "FSRQ", "FHFCZ", "FFR"}; "BZDM": 基金代码,
    "ABBNAME": 基金简称, "DJR": 权益登记日, "FSRQ": 除息日期, "FHFCZ": 分红(元/份), "FFR": 分红发放日
    :type rank: str
    :param sort: 排序方向；排序方式; choice of {"asc", "desc"}
    :type sort: str
    :param page: 查询页数；请求第page页数据; -1 表示全部页面
    :type page: int
    :return: 基金分红
    :rtype: pandas.DataFrame
    """

    def get_df_from_response(response):
        text = response.text
        return pd.DataFrame(eval(text[text.find("[["): text.find(";var jjfh_jjgs")]))

    url = "https://fund.eastmoney.com/Data/funddataIndex_Interface.aspx"
    params = {
        "dt": "8",
        "page": "1" if page == -1 else str(page),
        "rank": rank,
        "sort": sort,
        "gs": "",
        "ftype": typ,
        "year": year,
    }
    r = requests.get(url, params=params)
    data_list = [get_df_from_response(r)]
    if page == -1:
        data_text = r.text
        total_page = eval(data_text[data_text.find("=") + 1: data_text.find(";")])[0]
        tqdm = get_tqdm()
        for p in tqdm(range(2, total_page + 1), leave=False):
            params.update({"page": str(p)})
            r = requests.get(url, params=params)
            data_list.append(get_df_from_response(r))
    big_df = pd.concat(objs=data_list, ignore_index=True)
    big_df.reset_index(inplace=True)
    big_df["index"] = big_df.index + 1
    # 处理空数据时报错的问题
    if big_df.empty:
        big_df = big_df.reindex(
            columns=[
                "序号",
                "基金代码",
                "基金简称",
                "权益登记日",
                "除息日期",
                "分红",
                "分红发放日",
                "-",
            ]
        )
    big_df.columns = [
        "序号",
        "基金代码",
        "基金简称",
        "权益登记日",
        "除息日期",
        "分红",
        "分红发放日",
        "-",
    ]
    big_df = big_df[
        ["序号", "基金代码", "基金简称", "权益登记日", "除息日期", "分红", "分红发放日"]
    ]
    big_df["权益登记日"] = pd.to_datetime(big_df["权益登记日"]).dt.date
    big_df["除息日期"] = pd.to_datetime(big_df["除息日期"]).dt.date
    big_df["分红发放日"] = pd.to_datetime(big_df["分红发放日"]).dt.date
    big_df["分红"] = pd.to_numeric(big_df["分红"])
    return big_df


def fund_cf_em(
    year: str = "2025",
    typ: str = "",
    rank: str = "FSRQ",
    sort: str = "desc",
    page: int = -1,
) -> pd.DataFrame:
    """
    天天基金网-基金数据-分红送配-基金拆分
    https://fund.eastmoney.com/data/fundchaifen.html#FSRQ,desc,1,,,
    :param year: 查询年份
    :type year: str
    :param typ: 基金类型；空串表示全部; choice of {"", "指数型-其他", "指数型-海外股票", "指数型-固收", "指数型-股票",
    "债券型-中短债", "债券型-长债", "债券型-可转债", "债券型-混合债", "债券型-混合一级", "债券型-混合二级",
    "商品（不含QDII）", "货币型", "混合型-平衡", "混合型-偏债", "混合型-偏股", "混合型-灵活", "股票型", "QDII", "FOF"}
    :type typ: str
    :param rank: 排序字段；choice of {"BZDM", "ABBNAME", "FSRQ", "FHFCZ"}; "BZDM": 基金代码,
    "ABBNAME": 基金简称, "FSRQ": 拆分折算日, "FHFCZ": 拆分折算(每份)
    :type rank: str
    :param sort: 排序方向；choice of {"asc", "desc"}
    :type sort: str
    :param page: 查询页数；请求第page页数据; -1 表示全部页面
    :type page: int
    :return: 基金拆分
    :rtype: pandas.DataFrame
    """

    def get_df_from_response(response):
        text = response.text
        code = text[text.find("[["): text.find(";var jjcf_jjgs")]
        if code:
            return pd.DataFrame(eval(code))
        return pd.DataFrame()

    url = "https://fund.eastmoney.com/Data/funddataIndex_Interface.aspx"
    params = {
        "dt": "9",
        "page": "1" if page == -1 else str(page),
        "rank": rank,
        "sort": sort,
        "gs": "",
        "ftype": typ,
        "year": year,
    }
    r = requests.get(url, params=params)
    data_list = [get_df_from_response(r)]
    if page == -1:
        data_text = r.text
        total_page = eval(data_text[data_text.find("=") + 1: data_text.find(";")])[0]
        tqdm = get_tqdm()
        for p in tqdm(range(2, total_page + 1), leave=False):
            params.update({"page": str(p)})
            r = requests.get(url, params=params)
            data_list.append(get_df_from_response(r))
    big_df = pd.concat(objs=data_list, ignore_index=True)
    big_df.reset_index(inplace=True)
    big_df.loc[:, "index"] = big_df["index"] + 1
    # 处理空数据时报错的问题
    if big_df.empty:
        big_df = big_df.reindex(
            columns=[
                "序号",
                "基金代码",
                "基金简称",
                "拆分折算日",
                "拆分类型",
                "拆分折算",
                "-",
            ]
        )
    big_df.columns = [
        "序号",
        "基金代码",
        "基金简称",
        "拆分折算日",
        "拆分类型",
        "拆分折算",
        "-",
    ]
    big_df = big_df[
        ["序号", "基金代码", "基金简称", "拆分折算日", "拆分类型", "拆分折算"]
    ]
    big_df["拆分折算日"] = pd.to_datetime(big_df["拆分折算日"]).dt.date
    big_df["拆分折算"] = pd.to_numeric(big_df["拆分折算"], errors="coerce")
    return big_df


def fund_fh_rank_em() -> pd.DataFrame:
    """
    天天基金网-基金数据-分红送配-基金分红排行
    https://fund.eastmoney.com/data/fundleijifenhong.html
    :return: 基金分红排行
    :rtype: pandas.DataFrame
    """
    url = "https://fund.eastmoney.com/Data/funddataIndex_Interface.aspx"
    params = {
        "dt": "10",
        "page": "1",
        "rank": "FHFCZ",
        "sort": "desc",
        "gs": "",
        "ftype": "",
    }
    r = requests.get(url, params=params)
    data_text = r.text
    total_page = eval(data_text[data_text.find("=") + 1: data_text.find(";")])[0]
    big_df = pd.DataFrame()
    tqdm = get_tqdm()
    for page in tqdm(range(1, total_page + 1), leave=False):
        params.update({"page": str(page)})
        r = requests.get(url, params=params)
        data_text = r.text
        temp_list = eval(
            data_text[data_text.find("[["): data_text.find(";var fhph_jjgs")]
        )
        temp_df = pd.DataFrame(temp_list)
        big_df = pd.concat(objs=[big_df, temp_df], ignore_index=True)

    big_df.reset_index(inplace=True)
    big_df["index"] = big_df.index + 1
    # 处理空数据时报错的问题
    if big_df.empty:
        big_df = big_df.reindex(
            columns=[
                "序号",
                "基金代码",
                "基金简称",
                "累计分红",
                "累计次数",
                "成立日期",
                "-",
            ]
        )
    big_df.columns = [
        "序号",
        "基金代码",
        "基金简称",
        "累计分红",
        "累计次数",
        "成立日期",
        "-",
    ]
    big_df = big_df[
        ["序号", "基金代码", "基金简称", "累计分红", "累计次数", "成立日期"]
    ]
    big_df["成立日期"] = pd.to_datetime(big_df["成立日期"]).dt.date
    big_df["累计分红"] = pd.to_numeric(big_df["累计分红"], errors="coerce")
    big_df["累计次数"] = pd.to_numeric(big_df["累计次数"], errors="coerce")
    return big_df


if __name__ == "__main__":
    fund_fh_em_df = fund_fh_em(year="2025")
    print(fund_fh_em_df)

    fund_cf_em_df = fund_cf_em(year="2025")
    print(fund_cf_em_df)

    fund_fh_rank_em_df = fund_fh_rank_em()
    print(fund_fh_rank_em_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2025/6/23 15:00
Desc: 东方财富网-行情中心-外汇市场-所有汇率
https://quote.eastmoney.com/center/gridlist.html#forex_all
"""

import pandas as pd
import requests

from akshare.forex.cons import symbol_market_map
from akshare.utils.func import fetch_paginated_data


def forex_spot_em() -> pd.DataFrame:
    """
    东方财富网-行情中心-外汇市场-所有汇率-实时行情数据
    https://quote.eastmoney.com/center/gridlist.html#forex_all
    :return: 实时行情数据
    :rtype: pandas.DataFrame
    """
    url = "https://push2.eastmoney.com/api/qt/clist/get"
    params = {
        "np": "1",
        "fltt": "2",
        "invt": "2",
        "fs": "m:119,m:120,m:133",
        "fields": "f12,f13,f14,f1,f2,f4,f3,f152,f17,f18,f15,f16",
        "fid": "f3",
        "pn": "1",
        "pz": "100",
        "po": "1",
        "dect": "1",
        "wbp2u": "|0|0|0|web",
    }
    temp_df = fetch_paginated_data(url, params)
    temp_df.rename(
        columns={
            "index": "序号",
            "f12": "代码",
            "f14": "名称",
            "f17": "今开",
            "f4": "涨跌额",
            "f3": "涨跌幅",
            "f2": "最新价",
            "f15": "最高",
            "f16": "最低",
            "f18": "昨收",
        },
        inplace=True,
    )
    temp_df = temp_df[
        [
            "序号",
            "代码",
            "名称",
            "最新价",
            "涨跌额",
            "涨跌幅",
            "今开",
            "最高",
            "最低",
            "昨收",
        ]
    ]
    temp_df["最新价"] = pd.to_numeric(temp_df["最新价"], errors="coerce")
    temp_df["涨跌额"] = pd.to_numeric(temp_df["涨跌额"], errors="coerce")
    temp_df["涨跌幅"] = pd.to_numeric(temp_df["涨跌幅"], errors="coerce")
    temp_df["今开"] = pd.to_numeric(temp_df["今开"], errors="coerce")
    temp_df["最高"] = pd.to_numeric(temp_df["最高"], errors="coerce")
    temp_df["最低"] = pd.to_numeric(temp_df["最低"], errors="coerce")
    temp_df["昨收"] = pd.to_numeric(temp_df["昨收"], errors="coerce")
    return temp_df


def forex_hist_em(symbol: str = "USDCNH") -> pd.DataFrame:
    """
    东方财富网-行情中心-外汇市场-所有汇率-历史行情数据
    https://quote.eastmoney.com/cnyrate/EURCNYC.html
    :param symbol: 品种代码；可以通过 ak.forex_spot_em() 来获取所有可获取历史行情数据的品种代码
    :type symbol: str
    :return: 历史行情数据
    :rtype: pandas.DataFrame
    """
    url = "https://push2his.eastmoney.com/api/qt/stock/kline/get"
    market_code = symbol_market_map[symbol]
    params = {
        "secid": f"{market_code}.{symbol}",
        "klt": "101",
        "fqt": "1",
        "lmt": "50000",
        "end": "20500000",
        "iscca": "1",
        "fields1": "f1,f2,f3,f4,f5,f6,f7,f8",
        "fields2": "f51,f52,f53,f54,f55,f56,f57,f58,f59,f60,f61,f62,f63,f64",
        "ut": "f057cbcbce2a86e2866ab8877db1d059",
        "forcect": 1,
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame([item.split(",") for item in data_json["data"]["klines"]])
    temp_df["code"] = data_json["data"]["code"]
    temp_df["name"] = data_json["data"]["name"]
    temp_df.columns = [
        "日期",
        "今开",
        "最新价",
        "最高",
        "最低",
        "-",
        "-",
        "振幅",
        "-",
        "-",
        "-",
        "-",
        "-",
        "-",
        "代码",
        "名称",
    ]
    temp_df = temp_df[
        [
            "日期",
            "代码",
            "名称",
            "今开",
            "最新价",
            "最高",
            "最低",
            "振幅",
        ]
    ]
    temp_df["日期"] = pd.to_datetime(temp_df["日期"], errors="coerce").dt.date
    temp_df["今开"] = pd.to_numeric(temp_df["今开"], errors="coerce")
    temp_df["最新价"] = pd.to_numeric(temp_df["最新价"], errors="coerce")
    temp_df["最高"] = pd.to_numeric(temp_df["最高"], errors="coerce")
    temp_df["最低"] = pd.to_numeric(temp_df["最低"], errors="coerce")
    temp_df["振幅"] = pd.to_numeric(temp_df["振幅"], errors="coerce")
    return temp_df


if __name__ == "__main__":
    forex_spot_em_df = forex_spot_em()
    print(forex_spot_em_df)

    forex_hist_em_df = forex_hist_em(symbol="USDCNH")
    print(forex_hist_em_df)



# -*- coding:utf-8 -*-
# !/usr/bin/env python
"""
Date: 2024/9/21 19:00
Desc: 同花顺-数据中心-技术选股
https://data.10jqka.com.cn/rank/cxg/
"""

from io import StringIO

import pandas as pd
import py_mini_racer
import requests
from bs4 import BeautifulSoup

from akshare.datasets import get_ths_js
from akshare.utils.tqdm import get_tqdm


def _get_file_content_ths(file: str = "ths.js") -> str:
    """
    获取 JS 文件的内容
    :param file:  JS 文件名
    :type file: str
    :return: 文件内容
    :rtype: str
    """
    setting_file_path = get_ths_js(file)
    with open(setting_file_path, encoding="utf-8") as f:
        file_data = f.read()
    return file_data


def stock_rank_cxg_ths(symbol: str = "创月新高") -> pd.DataFrame:
    """
    同花顺-数据中心-技术选股-创新高
    https://data.10jqka.com.cn/rank/cxg/
    :param symbol: choice of {"创月新高", "半年新高", "一年新高", "历史新高"}
    :type symbol: str
    :return: 创新高数据
    :rtype: pandas.DataFrame
    """
    symbol_map = {
        "创月新高": "4",
        "半年新高": "3",
        "一年新高": "2",
        "历史新高": "1",
    }
    js_code = py_mini_racer.MiniRacer()
    js_content = _get_file_content_ths("ths.js")
    js_code.eval(js_content)
    v_code = js_code.call("v")
    headers = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 "
        "(KHTML, like Gecko) Chrome/89.0.4389.90 Safari/537.36",
        "Cookie": f"v={v_code}",
    }
    url = (
        f"http://data.10jqka.com.cn/rank/cxg/board/{symbol_map[symbol]}/field/"
        f"stockcode/order/asc/page/1/ajax/1/free/1/"
    )
    r = requests.get(url, headers=headers)
    soup = BeautifulSoup(r.text, features="lxml")
    try:
        total_page = soup.find(name="span", attrs={"class": "page_info"}).text.split(
            "/"
        )[1]
    except AttributeError:
        total_page = 1
    big_df = pd.DataFrame()
    tqdm = get_tqdm()
    for page in tqdm(range(1, int(total_page) + 1), leave=False):
        v_code = js_code.call("v")
        headers = {
            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 "
            "(KHTML, like Gecko) Chrome/89.0.4389.90 Safari/537.36",
            "Cookie": f"v={v_code}",
        }
        url = (
            f"http://data.10jqka.com.cn/rank/cxg/board/{symbol_map[symbol]}/field/stockcode/"
            f"order/asc/page/{page}/ajax/1/free/1/"
        )
        r = requests.get(url, headers=headers)
        temp_df = pd.read_html(StringIO(r.text))[0].iloc[:, :-1]  # 20260214 新增
        big_df = pd.concat(objs=[big_df, temp_df], ignore_index=True)
    big_df.columns = [
        "序号",
        "股票代码",
        "股票简称",
        "涨跌幅",
        "换手率",
        "最新价",
        "前期高点",
        "前期高点日期",
    ]
    big_df["股票代码"] = big_df["股票代码"].astype(str).str.zfill(6)
    big_df["涨跌幅"] = big_df["涨跌幅"].str.strip("%")
    big_df["换手率"] = big_df["换手率"].str.strip("%")
    big_df["前期高点日期"] = pd.to_datetime(
        big_df["前期高点日期"], errors="coerce"
    ).dt.date
    big_df["涨跌幅"] = pd.to_numeric(big_df["涨跌幅"], errors="coerce")
    big_df["换手率"] = pd.to_numeric(big_df["换手率"], errors="coerce")
    big_df["最新价"] = pd.to_numeric(big_df["最新价"], errors="coerce")
    big_df["前期高点"] = pd.to_numeric(big_df["前期高点"], errors="coerce")
    return big_df


def stock_rank_cxd_ths(symbol: str = "创月新低") -> pd.DataFrame:
    """
    同花顺-数据中心-技术选股-创新低
    https://data.10jqka.com.cn/rank/cxd/
    :param symbol: choice of {"创月新低", "半年新低", "一年新低", "历史新低"}
    :type symbol: str
    :return: 创新低数据
    :rtype: pandas.DataFrame
    """
    symbol_map = {
        "创月新低": "4",
        "半年新低": "3",
        "一年新低": "2",
        "历史新低": "1",
    }
    js_code = py_mini_racer.MiniRacer()
    js_content = _get_file_content_ths("ths.js")
    js_code.eval(js_content)
    v_code = js_code.call("v")
    headers = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 "
        "(KHTML, like Gecko) Chrome/89.0.4389.90 Safari/537.36",
        "Cookie": f"v={v_code}",
    }
    url = (
        f"http://data.10jqka.com.cn/rank/cxd/board/{symbol_map[symbol]}/field/"
        f"stockcode/order/asc/page/1/ajax/1/free/1/"
    )
    r = requests.get(url, headers=headers)
    soup = BeautifulSoup(r.text, features="lxml")
    try:
        total_page = soup.find(name="span", attrs={"class": "page_info"}).text.split(
            "/"
        )[1]
    except AttributeError:
        total_page = 1
    big_df = pd.DataFrame()
    tqdm = get_tqdm()
    for page in tqdm(range(1, int(total_page) + 1), leave=False):
        v_code = js_code.call("v")
        headers = {
            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 "
            "(KHTML, like Gecko) Chrome/89.0.4389.90 Safari/537.36",
            "Cookie": f"v={v_code}",
        }
        url = (
            f"http://data.10jqka.com.cn/rank/cxd/board/{symbol_map[symbol]}/field/"
            f"stockcode/order/asc/page/{page}/ajax/1/free/1/"
        )
        r = requests.get(url, headers=headers)
        temp_df = pd.read_html(StringIO(r.text))[0].iloc[:, :-1]  # 20260214 新增
        big_df = pd.concat(objs=[big_df, temp_df], ignore_index=True)
    big_df.columns = [
        "序号",
        "股票代码",
        "股票简称",
        "涨跌幅",
        "换手率",
        "最新价",
        "前期低点",
        "前期低点日期",
    ]
    big_df["股票代码"] = big_df["股票代码"].astype(str).str.zfill(6)
    big_df["涨跌幅"] = big_df["涨跌幅"].str.strip("%")
    big_df["换手率"] = big_df["换手率"].str.strip("%")
    big_df["前期低点日期"] = pd.to_datetime(
        big_df["前期低点日期"], errors="coerce"
    ).dt.date
    big_df["涨跌幅"] = pd.to_numeric(big_df["涨跌幅"], errors="coerce")
    big_df["换手率"] = pd.to_numeric(big_df["换手率"], errors="coerce")
    big_df["最新价"] = pd.to_numeric(big_df["最新价"], errors="coerce")
    big_df["前期低点"] = pd.to_numeric(big_df["前期低点"], errors="coerce")
    return big_df


def stock_rank_lxsz_ths() -> pd.DataFrame:
    """
    同花顺-数据中心-技术选股-连续上涨
    https://data.10jqka.com.cn/rank/lxsz/
    :return: 连续上涨
    :rtype: pandas.DataFrame
    """
    js_code = py_mini_racer.MiniRacer()
    js_content = _get_file_content_ths("ths.js")
    js_code.eval(js_content)
    v_code = js_code.call("v")
    headers = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 "
        "(KHTML, like Gecko) Chrome/89.0.4389.90 Safari/537.36",
        "Cookie": f"v={v_code}",
    }
    url = "http://data.10jqka.com.cn/rank/lxsz/field/lxts/order/desc/page/1/ajax/1/free/1/"
    r = requests.get(url, headers=headers)
    soup = BeautifulSoup(r.text, features="lxml")
    try:
        total_page = soup.find(name="span", attrs={"class": "page_info"}).text.split(
            "/"
        )[1]
    except AttributeError:
        total_page = 1
    big_df = pd.DataFrame()
    tqdm = get_tqdm()
    for page in tqdm(range(1, int(total_page) + 1), leave=False):
        v_code = js_code.call("v")
        headers = {
            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 "
            "(KHTML, like Gecko) Chrome/89.0.4389.90 Safari/537.36",
            "Cookie": f"v={v_code}",
        }
        url = f"http://data.10jqka.com.cn/rank/lxsz/field/lxts/order/desc/page/{page}/ajax/1/free/1/"
        r = requests.get(url, headers=headers)
        temp_df = pd.read_html(StringIO(r.text), converters={"股票代码": str})[0]
        big_df = pd.concat(objs=[big_df, temp_df], ignore_index=True)
    big_df.columns = [
        "序号",
        "股票代码",
        "股票简称",
        "收盘价",
        "最高价",
        "最低价",
        "连涨天数",
        "连续涨跌幅",
        "累计换手率",
        "所属行业",
    ]
    big_df["连续涨跌幅"] = big_df["连续涨跌幅"].str.strip("%")
    big_df["累计换手率"] = big_df["累计换手率"].str.strip("%")
    big_df["连续涨跌幅"] = pd.to_numeric(big_df["连续涨跌幅"], errors="coerce")
    big_df["累计换手率"] = pd.to_numeric(big_df["累计换手率"], errors="coerce")
    big_df["收盘价"] = pd.to_numeric(big_df["收盘价"], errors="coerce")
    big_df["最高价"] = pd.to_numeric(big_df["最高价"], errors="coerce")
    big_df["最低价"] = pd.to_numeric(big_df["最低价"], errors="coerce")
    big_df["连涨天数"] = pd.to_numeric(big_df["连涨天数"], errors="coerce")
    return big_df


def stock_rank_lxxd_ths() -> pd.DataFrame:
    """
    同花顺-数据中心-技术选股-连续下跌
    https://data.10jqka.com.cn/rank/lxxd/
    :return: 连续下跌
    :rtype: pandas.DataFrame
    """
    js_code = py_mini_racer.MiniRacer()
    js_content = _get_file_content_ths("ths.js")
    js_code.eval(js_content)
    v_code = js_code.call("v")
    headers = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 "
        "(KHTML, like Gecko) Chrome/89.0.4389.90 Safari/537.36",
        "Cookie": f"v={v_code}",
    }
    url = "http://data.10jqka.com.cn/rank/lxxd/field/lxts/order/desc/page/1/ajax/1/free/1/"
    r = requests.get(url, headers=headers)
    soup = BeautifulSoup(r.text, features="lxml")
    try:
        total_page = soup.find(name="span", attrs={"class": "page_info"}).text.split(
            "/"
        )[1]
    except AttributeError:
        total_page = 1
    big_df = pd.DataFrame()
    tqdm = get_tqdm()
    for page in tqdm(range(1, int(total_page) + 1), leave=False):
        v_code = js_code.call("v")
        headers = {
            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 "
            "(KHTML, like Gecko) Chrome/89.0.4389.90 Safari/537.36",
            "Cookie": f"v={v_code}",
        }
        url = f"http://data.10jqka.com.cn/rank/lxxd/field/lxts/order/desc/page/{page}/ajax/1/free/1/"
        r = requests.get(url, headers=headers)
        temp_df = pd.read_html(StringIO(r.text), converters={"股票代码": str})[0]
        big_df = pd.concat(objs=[big_df, temp_df], ignore_index=True)
    big_df.columns = [
        "序号",
        "股票代码",
        "股票简称",
        "收盘价",
        "最高价",
        "最低价",
        "连涨天数",
        "连续涨跌幅",
        "累计换手率",
        "所属行业",
    ]
    big_df["连续涨跌幅"] = big_df["连续涨跌幅"].str.strip("%")
    big_df["累计换手率"] = big_df["累计换手率"].str.strip("%")
    big_df["连续涨跌幅"] = pd.to_numeric(big_df["连续涨跌幅"], errors="coerce")
    big_df["累计换手率"] = pd.to_numeric(big_df["累计换手率"], errors="coerce")
    big_df["收盘价"] = pd.to_numeric(big_df["收盘价"], errors="coerce")
    big_df["最高价"] = pd.to_numeric(big_df["最高价"], errors="coerce")
    big_df["最低价"] = pd.to_numeric(big_df["最低价"], errors="coerce")
    big_df["连涨天数"] = pd.to_numeric(big_df["连涨天数"], errors="coerce")
    return big_df


def stock_rank_cxfl_ths() -> pd.DataFrame:
    """
    同花顺-数据中心-技术选股-持续放量
    https://data.10jqka.com.cn/rank/cxfl/
    :return: 持续放量
    :rtype: pandas.DataFrame
    """
    js_code = py_mini_racer.MiniRacer()
    js_content = _get_file_content_ths("ths.js")
    js_code.eval(js_content)
    v_code = js_code.call("v")
    headers = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 "
        "(KHTML, like Gecko) Chrome/89.0.4389.90 Safari/537.36",
        "Cookie": f"v={v_code}",
    }
    url = "http://data.10jqka.com.cn/rank/cxfl/field/count/order/desc/ajax/1/free/1/page/1/free/1/"
    r = requests.get(url, headers=headers)
    soup = BeautifulSoup(r.text, features="lxml")
    try:
        total_page = soup.find(name="span", attrs={"class": "page_info"}).text.split(
            "/"
        )[1]
    except AttributeError:
        total_page = 1
    big_df = pd.DataFrame()
    tqdm = get_tqdm()
    for page in tqdm(range(1, int(total_page) + 1), leave=False):
        v_code = js_code.call("v")
        headers = {
            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 "
            "(KHTML, like Gecko) Chrome/89.0.4389.90 Safari/537.36",
            "Cookie": f"v={v_code}",
        }
        url = f"http://data.10jqka.com.cn/rank/cxfl/field/count/order/desc/ajax/1/free/1/page/{page}/free/1/"
        r = requests.get(url, headers=headers)
        soup = BeautifulSoup(r.text, 'html.parser')
        table = soup.find('table', class_='m-table J-ajax-table')
        if not table:
            print("未找到表格")
            exit()

        rows = table.find('tbody').find_all('tr')
        data = []

        for row in rows:
            cols = row.find_all('td')
            if len(cols) >= 10:  # 确保是数据行
                item = {
                    '序号': cols[0].text.strip(),
                    '股票代码': cols[1].find('a').text.strip() if cols[1].find('a') else cols[1].text.strip(),
                    '股票简称': cols[2].find('a').text.strip() if cols[2].find('a') else cols[2].text.strip(),
                    '涨跌幅': cols[3].text.strip(),
                    '最新价': cols[4].text.strip(),
                    '成交量': cols[5].text.strip(),
                    '基准日成交量': cols[6].text.strip(),
                    '放量天数': cols[7].text.strip(),
                    '阶段涨跌幅': cols[8].text.strip(),
                    '所属行业': cols[9].find('a').text.strip() if cols[9].find('a') else cols[9].text.strip()
                }
                data.append(item)

        temp_df = pd.DataFrame(data)
        big_df = pd.concat(objs=[big_df, temp_df], ignore_index=True)
    big_df.columns = [
        "序号",
        "股票代码",
        "股票简称",
        "涨跌幅",
        "最新价",
        "成交量",
        "基准日成交量",
        "放量天数",
        "阶段涨跌幅",
        "所属行业",
    ]
    big_df["股票代码"] = big_df["股票代码"].astype(str).str.zfill(6)
    big_df["涨跌幅"] = big_df["涨跌幅"].astype(str).str.strip("%")
    big_df["阶段涨跌幅"] = big_df["阶段涨跌幅"].astype(str).str.strip("%")
    big_df["涨跌幅"] = pd.to_numeric(big_df["涨跌幅"], errors="coerce")
    big_df["阶段涨跌幅"] = pd.to_numeric(big_df["阶段涨跌幅"], errors="coerce")
    big_df["最新价"] = pd.to_numeric(big_df["最新价"], errors="coerce")
    big_df["放量天数"] = pd.to_numeric(big_df["放量天数"], errors="coerce")
    return big_df


def stock_rank_cxsl_ths() -> pd.DataFrame:
    """
    同花顺-数据中心-技术选股-持续缩量
    https://data.10jqka.com.cn/rank/cxsl/
    :return: 持续缩量
    :rtype: pandas.DataFrame
    """
    js_code = py_mini_racer.MiniRacer()
    js_content = _get_file_content_ths("ths.js")
    js_code.eval(js_content)
    v_code = js_code.call("v")
    headers = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 "
        "(KHTML, like Gecko) Chrome/89.0.4389.90 Safari/537.36",
        "Cookie": f"v={v_code}",
    }
    url = "http://data.10jqka.com.cn/rank/cxsl/field/count/order/desc/ajax/1/free/1/page/1/free/1/"
    r = requests.get(url, headers=headers)
    soup = BeautifulSoup(r.text, features="lxml")
    try:
        total_page = soup.find(name="span", attrs={"class": "page_info"}).text.split(
            "/"
        )[1]
    except AttributeError:
        total_page = 1
    big_df = pd.DataFrame()
    tqdm = get_tqdm()
    for page in tqdm(range(1, int(total_page) + 1), leave=False):
        v_code = js_code.call("v")
        headers = {
            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 "
            "(KHTML, like Gecko) Chrome/89.0.4389.90 Safari/537.36",
            "Cookie": f"v={v_code}",
        }
        url = f"http://data.10jqka.com.cn/rank/cxsl/field/count/order/desc/ajax/1/free/1/page/{page}/free/1/"
        r = requests.get(url, headers=headers)
        soup = BeautifulSoup(r.text, 'html.parser')
        table = soup.find('table', class_='m-table J-ajax-table')
        if not table:
            print("未找到表格")
            exit()

        rows = table.find('tbody').find_all('tr')
        data = []

        for row in rows:
            cols = row.find_all('td')
            if len(cols) >= 10:  # 确保是数据行
                item = {
                    '序号': cols[0].text.strip(),
                    '股票代码': cols[1].find('a').text.strip() if cols[1].find('a') else cols[1].text.strip(),
                    '股票简称': cols[2].find('a').text.strip() if cols[2].find('a') else cols[2].text.strip(),
                    '涨跌幅': cols[3].text.strip(),
                    '最新价': cols[4].text.strip(),
                    '成交量': cols[5].text.strip(),
                    '基准日成交量': cols[6].text.strip(),
                    '放量天数': cols[7].text.strip(),
                    '阶段涨跌幅': cols[8].text.strip(),
                    '所属行业': cols[9].find('a').text.strip() if cols[9].find('a') else cols[9].text.strip()
                }
                data.append(item)

        temp_df = pd.DataFrame(data)
        big_df = pd.concat(objs=[big_df, temp_df], ignore_index=True)
    big_df.columns = [
        "序号",
        "股票代码",
        "股票简称",
        "涨跌幅",
        "最新价",
        "成交量",
        "基准日成交量",
        "缩量天数",
        "阶段涨跌幅",
        "所属行业",
    ]
    big_df["股票代码"] = big_df["股票代码"].astype(str).str.zfill(6)
    big_df["涨跌幅"] = big_df["涨跌幅"].astype(str).str.strip("%")
    big_df["阶段涨跌幅"] = big_df["阶段涨跌幅"].astype(str).str.strip("%")
    big_df["涨跌幅"] = pd.to_numeric(big_df["涨跌幅"], errors="coerce")
    big_df["阶段涨跌幅"] = pd.to_numeric(big_df["阶段涨跌幅"], errors="coerce")
    big_df["最新价"] = pd.to_numeric(big_df["最新价"], errors="coerce")
    big_df["缩量天数"] = pd.to_numeric(big_df["缩量天数"], errors="coerce")
    return big_df


def stock_rank_xstp_ths(symbol: str = "500日均线") -> pd.DataFrame:
    """
    同花顺-数据中心-技术选股-向上突破
    https://data.10jqka.com.cn/rank/xstp/
    :param symbol: choice of {"5日均线", "10日均线", "20日均线", "30日均线", "60日均线", "90日均线", "250日均线", "500日均线"}
    :type symbol: str
    :return: 向上突破
    :rtype: pandas.DataFrame
    """
    symbol_map = {
        "5日均线": 5,
        "10日均线": 10,
        "20日均线": 20,
        "30日均线": 30,
        "60日均线": 60,
        "90日均线": 90,
        "250日均线": 250,
        "500日均线": 500,
    }
    js_code = py_mini_racer.MiniRacer()
    js_content = _get_file_content_ths("ths.js")
    js_code.eval(js_content)
    v_code = js_code.call("v")
    headers = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 "
        "(KHTML, like Gecko) Chrome/89.0.4389.90 Safari/537.36",
        "Cookie": f"v={v_code}",
    }
    url = f"http://data.10jqka.com.cn/rank/xstp/board/{symbol_map[symbol]}/order/asc/ajax/1/free/1/page/1/free/1/"
    r = requests.get(url, headers=headers)
    soup = BeautifulSoup(r.text, features="lxml")
    try:
        total_page = soup.find(name="span", attrs={"class": "page_info"}).text.split(
            "/"
        )[1]
    except AttributeError:
        total_page = 1
    big_df = pd.DataFrame()
    tqdm = get_tqdm()
    for page in tqdm(range(1, int(total_page) + 1), leave=False):
        v_code = js_code.call("v")
        headers = {
            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 "
            "(KHTML, like Gecko) Chrome/89.0.4389.90 Safari/537.36",
            "Cookie": f"v={v_code}",
        }
        url = (
            f"http://data.10jqka.com.cn/rank/xstp/board/{symbol_map[symbol]}/order/"
            f"asc/ajax/1/free/1/page/{page}/free/1/"
        )
        r = requests.get(url, headers=headers)
        pd.read_html(StringIO(r.text))
        soup = BeautifulSoup(r.text, 'html.parser')
        table = soup.find('table', class_='m-table J-ajax-table')
        if not table:
            print("未找到表格")
            exit()

        rows = table.find('tbody').find_all('tr')
        data = []

        for row in rows:
            cols = row.find_all('td')
            if len(cols) >= 8:  # 确保是数据行
                item = {
                    '序号': cols[0].text.strip(),
                    '股票代码': cols[1].find('a').text.strip() if cols[1].find('a') else cols[1].text.strip(),
                    '股票简称': cols[2].find('a').text.strip() if cols[2].find('a') else cols[2].text.strip(),
                    '最新价': cols[3].text.strip(),
                    '成交额': cols[4].text.strip(),
                    '成交量': cols[5].text.strip(),
                    '涨跌幅': cols[6].text.strip(),
                    '换手率': cols[7].text.strip(),
                }
                data.append(item)

        temp_df = pd.DataFrame(data)
        big_df = pd.concat(objs=[big_df, temp_df], ignore_index=True)
    big_df.columns = [
        "序号",
        "股票代码",
        "股票简称",
        "最新价",
        "成交额",
        "成交量",
        "涨跌幅",
        "换手率",
    ]
    big_df["股票代码"] = big_df["股票代码"].astype(str).str.zfill(6)
    big_df["涨跌幅"] = big_df["涨跌幅"].astype(str).str.strip("%")
    big_df["换手率"] = big_df["换手率"].astype(str).str.strip("%")
    big_df["涨跌幅"] = pd.to_numeric(big_df["涨跌幅"], errors="coerce")
    big_df["换手率"] = pd.to_numeric(big_df["换手率"], errors="coerce")
    big_df["最新价"] = pd.to_numeric(big_df["最新价"], errors="coerce")
    return big_df


def stock_rank_xxtp_ths(symbol: str = "500日均线") -> pd.DataFrame:
    """
    同花顺-数据中心-技术选股-向下突破
    https://data.10jqka.com.cn/rank/xxtp/
    :param symbol: choice of {"5日均线", "10日均线", "20日均线", "30日均线", "60日均线", "90日均线", "250日均线", "500日均线"}
    :type symbol: str
    :return: 向下突破
    :rtype: pandas.DataFrame
    """
    symbol_map = {
        "5日均线": 5,
        "10日均线": 10,
        "20日均线": 20,
        "30日均线": 30,
        "60日均线": 60,
        "90日均线": 90,
        "250日均线": 250,
        "500日均线": 500,
    }
    js_code = py_mini_racer.MiniRacer()
    js_content = _get_file_content_ths("ths.js")
    js_code.eval(js_content)
    v_code = js_code.call("v")
    headers = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 "
        "(KHTML, like Gecko) Chrome/89.0.4389.90 Safari/537.36",
        "Cookie": f"v={v_code}",
    }
    url = f"http://data.10jqka.com.cn/rank/xxtp/board/{symbol_map[symbol]}/order/asc/ajax/1/free/1/page/1/free/1/"
    r = requests.get(url, headers=headers)
    soup = BeautifulSoup(r.text, features="lxml")
    try:
        total_page = soup.find(name="span", attrs={"class": "page_info"}).text.split(
            "/"
        )[1]
    except AttributeError:
        total_page = 1
    big_df = pd.DataFrame()
    tqdm = get_tqdm()
    for page in tqdm(range(1, int(total_page) + 1), leave=False):
        v_code = js_code.call("v")
        headers = {
            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 "
            "(KHTML, like Gecko) Chrome/89.0.4389.90 Safari/537.36",
            "Cookie": f"v={v_code}",
        }
        url = (
            f"http://data.10jqka.com.cn/rank/xxtp/board/{symbol_map[symbol]}/order/"
            f"asc/ajax/1/free/1/page/{page}/free/1/"
        )
        r = requests.get(url, headers=headers)
        soup = BeautifulSoup(r.text, 'html.parser')
        table = soup.find('table', class_='m-table J-ajax-table')
        if not table:
            print("未找到表格")
            exit()

        rows = table.find('tbody').find_all('tr')
        data = []

        for row in rows:
            cols = row.find_all('td')
            if len(cols) >= 8:  # 确保是数据行
                item = {
                    '序号': cols[0].text.strip(),
                    '股票代码': cols[1].find('a').text.strip() if cols[1].find('a') else cols[1].text.strip(),
                    '股票简称': cols[2].find('a').text.strip() if cols[2].find('a') else cols[2].text.strip(),
                    '最新价': cols[3].text.strip(),
                    '成交额': cols[4].text.strip(),
                    '成交量': cols[5].text.strip(),
                    '涨跌幅': cols[6].text.strip(),
                    '换手率': cols[7].text.strip(),
                }
                data.append(item)

        temp_df = pd.DataFrame(data)
        big_df = pd.concat(objs=[big_df, temp_df], ignore_index=True)
    big_df.columns = [
        "序号",
        "股票代码",
        "股票简称",
        "最新价",
        "成交额",
        "成交量",
        "涨跌幅",
        "换手率",
    ]
    big_df["股票代码"] = big_df["股票代码"].astype(str).str.zfill(6)
    big_df["涨跌幅"] = big_df["涨跌幅"].astype(str).str.strip("%")
    big_df["换手率"] = big_df["换手率"].astype(str).str.strip("%")
    big_df["涨跌幅"] = pd.to_numeric(big_df["涨跌幅"], errors="coerce")
    big_df["换手率"] = pd.to_numeric(big_df["换手率"], errors="coerce")
    big_df["最新价"] = pd.to_numeric(big_df["最新价"], errors="coerce")
    return big_df


def stock_rank_ljqs_ths() -> pd.DataFrame:
    """
    同花顺-数据中心-技术选股-量价齐升
    http://data.10jqka.com.cn/rank/ljqs/
    :return: 量价齐升
    :rtype: pandas.DataFrame
    """
    js_code = py_mini_racer.MiniRacer()
    js_content = _get_file_content_ths("ths.js")
    js_code.eval(js_content)
    v_code = js_code.call("v")
    headers = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 "
        "(KHTML, like Gecko) Chrome/89.0.4389.90 Safari/537.36",
        "Cookie": f"v={v_code}",
    }
    url = "http://data.10jqka.com.cn/rank/ljqs/field/count/order/desc/ajax/1/free/1/page/1/free/1/"
    r = requests.get(url, headers=headers)
    soup = BeautifulSoup(r.text, features="lxml")
    try:
        total_page = soup.find(name="span", attrs={"class": "page_info"}).text.split(
            "/"
        )[1]
    except AttributeError:
        total_page = 1
    big_df = pd.DataFrame()
    tqdm = get_tqdm()
    for page in tqdm(range(1, int(total_page) + 1), leave=False):
        v_code = js_code.call("v")
        headers = {
            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 "
            "(KHTML, like Gecko) Chrome/89.0.4389.90 Safari/537.36",
            "Cookie": f"v={v_code}",
        }
        url = f"http://data.10jqka.com.cn/rank/ljqs/field/count/order/desc/ajax/1/free/1/page/{page}/free/1/"
        r = requests.get(url, headers=headers)
        soup = BeautifulSoup(r.text, 'html.parser')
        table = soup.find('table', class_='m-table J-ajax-table')
        if not table:
            print("未找到表格")
            exit()

        rows = table.find('tbody').find_all('tr')
        data = []

        for row in rows:
            cols = row.find_all('td')
            if len(cols) >= 8:  # 确保是数据行
                item = {
                    '序号': cols[0].text.strip(),
                    '股票代码': cols[1].find('a').text.strip() if cols[1].find('a') else cols[1].text.strip(),
                    '股票简称': cols[2].find('a').text.strip() if cols[2].find('a') else cols[2].text.strip(),
                    '最新价': cols[3].text.strip(),
                    '量价齐升天数': cols[4].text.strip(),
                    '阶段涨幅': cols[5].text.strip(),
                    '累计换手率': cols[6].text.strip(),
                    '所属行业': cols[7].find('a').text.strip() if cols[7].find('a') else cols[7].text.strip()
                }
                data.append(item)

        temp_df = pd.DataFrame(data)
        big_df = pd.concat(objs=[big_df, temp_df], ignore_index=True)
    big_df.columns = [
        "序号",
        "股票代码",
        "股票简称",
        "最新价",
        "量价齐升天数",
        "阶段涨幅",
        "累计换手率",
        "所属行业",
    ]
    big_df["股票代码"] = big_df["股票代码"].astype(str).str.zfill(6)
    big_df["阶段涨幅"] = big_df["阶段涨幅"].astype(str).str.strip("%")
    big_df["累计换手率"] = big_df["累计换手率"].astype(str).str.strip("%")
    big_df["阶段涨幅"] = pd.to_numeric(big_df["阶段涨幅"], errors="coerce")
    big_df["累计换手率"] = pd.to_numeric(big_df["累计换手率"], errors="coerce")
    big_df["最新价"] = pd.to_numeric(big_df["最新价"], errors="coerce")
    big_df["量价齐升天数"] = pd.to_numeric(big_df["量价齐升天数"], errors="coerce")
    return big_df


def stock_rank_ljqd_ths() -> pd.DataFrame:
    """
    同花顺-数据中心-技术选股-量价齐跌
    http://data.10jqka.com.cn/rank/ljqd/
    :return: 量价齐跌
    :rtype: pandas.DataFrame
    """
    js_code = py_mini_racer.MiniRacer()
    js_content = _get_file_content_ths("ths.js")
    js_code.eval(js_content)
    v_code = js_code.call("v")
    headers = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 "
        "(KHTML, like Gecko) Chrome/89.0.4389.90 Safari/537.36",
        "Cookie": f"v={v_code}",
    }
    url = "http://data.10jqka.com.cn/rank/ljqd/field/count/order/desc/ajax/1/free/1/page/1/free/1/"
    r = requests.get(url, headers=headers)
    soup = BeautifulSoup(r.text, features="lxml")
    try:
        total_page = soup.find(name="span", attrs={"class": "page_info"}).text.split(
            "/"
        )[1]
    except AttributeError:
        total_page = 1
    big_df = pd.DataFrame()
    tqdm = get_tqdm()
    for page in tqdm(range(1, int(total_page) + 1), leave=False):
        v_code = js_code.call("v")
        headers = {
            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 "
            "(KHTML, like Gecko) Chrome/89.0.4389.90 Safari/537.36",
            "Cookie": f"v={v_code}",
        }
        url = f"http://data.10jqka.com.cn/rank/ljqd/field/count/order/desc/ajax/1/free/1/page/{page}/free/1/"
        r = requests.get(url, headers=headers)
        soup = BeautifulSoup(r.text, 'html.parser')
        table = soup.find('table', class_='m-table J-ajax-table')
        if not table:
            print("未找到表格")
            exit()

        rows = table.find('tbody').find_all('tr')
        data = []

        for row in rows:
            cols = row.find_all('td')
            if len(cols) >= 8:  # 确保是数据行
                item = {
                    '序号': cols[0].text.strip(),
                    '股票代码': cols[1].find('a').text.strip() if cols[1].find('a') else cols[1].text.strip(),
                    '股票简称': cols[2].find('a').text.strip() if cols[2].find('a') else cols[2].text.strip(),
                    '最新价': cols[3].text.strip(),
                    '量价齐跌天数': cols[4].text.strip(),
                    '阶段涨幅': cols[5].text.strip(),
                    '累计换手率': cols[6].text.strip(),
                    '所属行业': cols[7].find('a').text.strip() if cols[7].find('a') else cols[7].text.strip()
                }
                data.append(item)

        temp_df = pd.DataFrame(data)
        big_df = pd.concat(objs=[big_df, temp_df], ignore_index=True)
    big_df.columns = [
        "序号",
        "股票代码",
        "股票简称",
        "最新价",
        "量价齐跌天数",
        "阶段涨幅",
        "累计换手率",
        "所属行业",
    ]
    big_df["股票代码"] = big_df["股票代码"].astype(str).str.zfill(6)
    big_df["阶段涨幅"] = big_df["阶段涨幅"].astype(str).str.strip("%")
    big_df["累计换手率"] = big_df["累计换手率"].astype(str).str.strip("%")
    big_df["阶段涨幅"] = pd.to_numeric(big_df["阶段涨幅"], errors="coerce")
    big_df["累计换手率"] = pd.to_numeric(big_df["累计换手率"], errors="coerce")
    big_df["最新价"] = pd.to_numeric(big_df["最新价"], errors="coerce")
    big_df["量价齐跌天数"] = pd.to_numeric(big_df["量价齐跌天数"], errors="coerce")
    return big_df


def stock_rank_xzjp_ths() -> pd.DataFrame:
    """
    同花顺-数据中心-技术选股-险资举牌
    https://data.10jqka.com.cn/financial/xzjp/
    :return: 险资举牌
    :rtype: pandas.DataFrame
    """
    js_code = py_mini_racer.MiniRacer()
    js_content = _get_file_content_ths("ths.js")
    js_code.eval(js_content)
    big_df = pd.DataFrame()
    v_code = js_code.call("v")
    headers = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 "
        "(KHTML, like Gecko) Chrome/89.0.4389.90 Safari/537.36",
        "Cookie": f"v={v_code}",
    }
    url = "http://data.10jqka.com.cn/ajax/xzjp/field/DECLAREDATE/order/desc/ajax/1/free/1/"
    r = requests.get(url, headers=headers)
    soup = BeautifulSoup(r.text, 'html.parser')
    table = soup.find('table', class_='m-table J-ajax-table')
    if not table:
        print("未找到表格")
        exit()

    rows = table.find('tbody').find_all('tr')
    data = []

    for row in rows:
        cols = row.find_all('td')
        if len(cols) >= 8:  # 确保是数据行
            item = {
                '序号': cols[0].text.strip(),
                '举牌公告日': cols[1].find('a').text.strip() if cols[1].find('a') else cols[1].text.strip(),
                '股票代码': cols[2].find('a').text.strip() if cols[2].find('a') else cols[2].text.strip(),
                '股票简称': cols[3].text.strip(),
                '现价': cols[4].text.strip(),
                '涨跌幅': cols[5].text.strip(),
                '举牌方': cols[6].text.strip(),
                '增持数量': cols[7].find('a').text.strip() if cols[7].find('a') else cols[7].text.strip(),
                '交易均价': cols[8].find('a').text.strip() if cols[8].find('a') else cols[8].text.strip(),
                '增持数量占总股本比例': cols[9].find('a').text.strip() if cols[9].find('a') else cols[9].text.strip(),
                '变动后持股总数': cols[10].find('a').text.strip() if cols[10].find('a') else cols[10].text.strip(),
                '变动后持股比例': cols[11].find('a').text.strip() if cols[11].find('a') else cols[11].text.strip(),
                '历史数据': cols[12].find('a').text.strip() if cols[12].find('a') else cols[12].text.strip(),
            }
            data.append(item)

    temp_df = pd.DataFrame(data)
    big_df = pd.concat(objs=[big_df, temp_df], ignore_index=True)
    big_df.columns = [
        "序号",
        "举牌公告日",
        "股票代码",
        "股票简称",
        "现价",
        "涨跌幅",
        "举牌方",
        "增持数量",
        "交易均价",
        "增持数量占总股本比例",
        "变动后持股总数",
        "变动后持股比例",
        "历史数据",
    ]
    big_df["涨跌幅"] = big_df["涨跌幅"].astype(str).str.zfill(6)
    big_df["增持数量占总股本比例"] = (
        big_df["增持数量占总股本比例"].astype(str).str.strip("%")
    )
    big_df["变动后持股比例"] = big_df["变动后持股比例"].astype(str).str.strip("%")
    big_df["涨跌幅"] = pd.to_numeric(big_df["涨跌幅"], errors="coerce")
    big_df["增持数量占总股本比例"] = pd.to_numeric(
        big_df["增持数量占总股本比例"], errors="coerce"
    )
    big_df["变动后持股比例"] = pd.to_numeric(big_df["变动后持股比例"], errors="coerce")
    big_df["举牌公告日"] = pd.to_datetime(big_df["举牌公告日"], errors="coerce").dt.date
    big_df["股票代码"] = big_df["股票代码"].astype(str).str.zfill(6)
    big_df["现价"] = pd.to_numeric(big_df["现价"], errors="coerce")
    big_df["交易均价"] = pd.to_numeric(big_df["交易均价"], errors="coerce")
    del big_df["历史数据"]
    return big_df


if __name__ == "__main__":
    stock_rank_cxg_ths_df = stock_rank_cxg_ths(symbol="创月新高")
    print(stock_rank_cxg_ths_df)

    stock_rank_cxg_ths_df = stock_rank_cxg_ths(symbol="半年新高")
    print(stock_rank_cxg_ths_df)

    stock_rank_cxg_ths_df = stock_rank_cxg_ths(symbol="一年新高")
    print(stock_rank_cxg_ths_df)

    stock_rank_cxg_ths_df = stock_rank_cxg_ths(symbol="历史新高")
    print(stock_rank_cxg_ths_df)

    stock_rank_cxd_ths_df = stock_rank_cxd_ths(symbol="创月新低")
    print(stock_rank_cxd_ths_df)

    stock_rank_cxd_ths_df = stock_rank_cxd_ths(symbol="半年新低")
    print(stock_rank_cxd_ths_df)

    stock_rank_cxd_ths_df = stock_rank_cxd_ths(symbol="一年新低")
    print(stock_rank_cxd_ths_df)

    stock_rank_cxd_ths_df = stock_rank_cxd_ths(symbol="历史新低")
    print(stock_rank_cxd_ths_df)

    stock_rank_lxsz_ths_df = stock_rank_lxsz_ths()
    print(stock_rank_lxsz_ths_df)

    stock_rank_lxxd_ths_df = stock_rank_lxxd_ths()
    print(stock_rank_lxxd_ths_df)

    stock_rank_cxfl_ths_df = stock_rank_cxfl_ths()
    print(stock_rank_cxfl_ths_df)

    stock_rank_cxsl_ths_df = stock_rank_cxsl_ths()
    print(stock_rank_cxsl_ths_df)

    stock_rank_xstp_ths_df = stock_rank_xstp_ths(symbol="5日均线")
    print(stock_rank_xstp_ths_df)

    stock_rank_xxtp_ths_df = stock_rank_xxtp_ths(symbol="5日均线")
    print(stock_rank_xxtp_ths_df)

    stock_rank_ljqs_ths_df = stock_rank_ljqs_ths()
    print(stock_rank_ljqs_ths_df)

    stock_rank_ljqd_ths_df = stock_rank_ljqd_ths()
    print(stock_rank_ljqd_ths_df)

    stock_rank_xzjp_ths_df = stock_rank_xzjp_ths()
    print(stock_rank_xzjp_ths_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2026/4/3 16:05
Desc: 乐咕乐股-股息率-A 股股息率
https://legulegu.com/stockdata/guxilv
"""

import pandas as pd
import requests

from akshare.stock_feature.stock_a_indicator import get_token_lg, get_cookie_csrf


def stock_a_gxl_lg(symbol: str = "上证A股") -> pd.DataFrame:
    """
    乐咕乐股-股息率-A 股股息率
    https://legulegu.com/stockdata/guxilv
    :param symbol: choice of {"上证A股", "深证A股", "创业板", "科创板"}
    :type symbol: str
    :return: A 股股息率
    :rtype: pandas.DataFrame
    """
    symbol_map = {
        "上证A股": "shangzheng",
        "深证A股": "shenzheng",
        "创业板": "chuangyeban",
        "科创板": "kechuangban",
    }
    url = "https://legulegu.com/api/stockdata/guxilv"
    token = get_token_lg()
    params = {"token": token}
    r = requests.get(
        url,
        params=params,
        **get_cookie_csrf(url="https://legulegu.com/stockdata/guxilv"),
    )
    data_json = r.json()
    temp_df = pd.DataFrame(data_json[symbol_map[symbol]])
    temp_df["date"] = (
        pd.to_datetime(temp_df["date"], utc=True)
        .dt.tz_convert("Asia/Shanghai")
        .dt.date
    )
    temp_df.rename(columns={"addDvTtm": "股息率", "date": "日期"}, inplace=True)
    temp_df = temp_df[
        [
            "日期",
            "股息率",
        ]
    ]
    temp_df["股息率"] = pd.to_numeric(temp_df["股息率"], errors="coerce")
    return temp_df


def stock_hk_gxl_lg() -> pd.DataFrame:
    """
    乐咕乐股-股息率-恒生指数股息率
    https://legulegu.com/stockdata/market/hk/dv/hsi
    :return: 恒生指数股息率
    :rtype: pandas.DataFrame
    """
    url = "https://legulegu.com/api/stockdata/hs"
    token = get_token_lg()
    params = {"token": token, "indexCode": "HSI"}
    r = requests.get(
        url,
        params=params,
        **get_cookie_csrf(url="https://legulegu.com/stockdata/market/hk/dv/hsi"),
    )
    data_json = r.json()
    temp_df = pd.DataFrame(data_json)
    temp_df["date"] = (
        pd.to_datetime(temp_df["date"], utc=True)
        .dt.tz_convert("Asia/Shanghai")
        .dt.date
    )
    temp_df.rename(columns={"dvRatio": "股息率", "date": "日期"}, inplace=True)
    temp_df = temp_df[
        [
            "日期",
            "股息率",
        ]
    ]
    temp_df["股息率"] = pd.to_numeric(temp_df["股息率"], errors="coerce")
    return temp_df


if __name__ == "__main__":
    stock_a_gxl_lg_df = stock_a_gxl_lg(symbol="上证A股")
    print(stock_a_gxl_lg_df)

    stock_hk_gxl_lg_df = stock_hk_gxl_lg()
    print(stock_hk_gxl_lg_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2025/03/04
Desc: 同花顺-数据中心-新股申购与中签
https://data.10jqka.com.cn/ipo/xgsgyzq/
"""

import pandas as pd
import requests
from bs4 import BeautifulSoup


def stock_ipo_ths(symbol: str = "全部A股") -> pd.DataFrame:
    """
    同花顺-数据中心-新股申购与中签
    https://data.10jqka.com.cn/ipo/xgsgyzq/
    :param symbol: choice of {"全部A股", "沪市主板", "深市主板", "创业板", "科创板", "京市主板"}
    :type symbol: str
    :return: 新股申购与中签数据
    :rtype: pandas.DataFrame
    """
    symbol_map = {
        "全部A股": "all",
        "沪市主板": "hszb",
        "深市主板": "sszb",
        "创业板": "cyb",
        "科创板": "kcbsg",
        "京市主板": "bjzb",
    }
    if symbol not in symbol_map:
        raise ValueError(
            f"Invalid symbol: {symbol}. "
            f"Please choose from {list(symbol_map.keys())}"
        )

    url = f"https://data.10jqka.com.cn/ipo/xgsgyzq/{symbol_map[symbol]}/"
    headers = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
                      "Chrome/89.0.4389.90 Safari/537.36",
    }
    r = requests.get(url, headers=headers)
    r.raise_for_status()
    soup = BeautifulSoup(r.text, "lxml")

    # 查找表格
    table = soup.find("table", id="maintable")
    if not table:
        table = soup.find("table", class_="m_table")
    if not table:
        raise ValueError("未找到数据表格，请检查网页结构是否发生变化")

    # 获取表头
    thead = table.find("thead")
    if thead:
        headers_list = [th.get_text(strip=True) for th in thead.find_all("th")]
    else:
        headers_list = []

    # 获取表格数据
    tbody = table.find("tbody")
    if not tbody:
        tbody = table  # 有些表格没有tbody标签

    data = []
    for row in tbody.find_all("tr"):
        cells = [td.get_text(strip=True) for td in row.find_all("td")]
        if cells:
            data.append(cells)

    # 创建DataFrame
    if headers_list and data:
        temp_df = pd.DataFrame(data, columns=headers_list)
    elif data:
        temp_df = pd.DataFrame(data)
    else:
        temp_df = pd.DataFrame()

    return temp_df


def stock_ipo_hk_ths() -> pd.DataFrame:
    """
    同花顺-数据中心-新股申购与中签-港股
    https://data.10jqka.com.cn/ipo/xgsgyzq/
    :return: 港股新股申购与中签数据
    :rtype: pandas.DataFrame
    """
    url = "https://data.10jqka.com.cn/ipo/xgsgyzq/hkstock/"
    headers = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
                      "Chrome/89.0.4389.90 Safari/537.36",
    }
    r = requests.get(url, headers=headers)
    r.raise_for_status()
    soup = BeautifulSoup(r.text, "lxml")

    # 查找表格
    table = soup.find("table", id="maintable")
    if not table:
        table = soup.find("table", class_="m_table")
    if not table:
        raise ValueError("未找到数据表格，请检查网页结构是否发生变化")

    # 获取表头
    thead = table.find("thead")
    if thead:
        headers_list = [th.get_text(strip=True) for th in thead.find_all("th")]
    else:
        headers_list = []

    # 获取表格数据
    tbody = table.find("tbody")
    if not tbody:
        tbody = table  # 有些表格没有tbody标签

    data = []
    for row in tbody.find_all("tr"):
        cells = [td.get_text(strip=True) for td in row.find_all("td")]
        if cells:
            data.append(cells)

    # 创建DataFrame
    if headers_list and data:
        temp_df = pd.DataFrame(data, columns=headers_list)
    elif data:
        temp_df = pd.DataFrame(data)
    else:
        temp_df = pd.DataFrame()

    return temp_df


if __name__ == "__main__":
    stock_ipo_ths_df = stock_ipo_ths(symbol="全部A股")
    print(stock_ipo_ths_df)

    stock_ipo_hk_ths_df = stock_ipo_hk_ths()
    print(stock_ipo_hk_ths_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2025/10/30 17:40
Desc: openctp-合约信息接口
http://openctp.cn/instruments.html
"""

import pandas as pd
import requests


def option_contract_info_ctp() -> pd.DataFrame:
    """
    openctp-合约信息接口-期权合约
    http://openctp.cn/instruments.html
    :return: 期权合约信息
    :rtype: pandas.DataFrame
    """
    url = "http://dict.openctp.cn/instruments?types=option"
    r = requests.get(url)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["data"])

    # 字段映射：英文字段名 -> 中文字段名
    column_mapping = {
        "ExchangeID": "交易所ID",
        "InstrumentID": "合约ID",
        "InstrumentName": "合约名称",
        "ProductClass": "商品类别",
        "ProductID": "品种ID",
        "VolumeMultiple": "合约乘数",
        "PriceTick": "最小变动价位",
        "LongMarginRatioByMoney": "做多保证金率",
        "ShortMarginRatioByMoney": "做空保证金率",
        "LongMarginRatioByVolume": "做多保证金/手",
        "ShortMarginRatioByVolume": "做空保证金/手",
        "OpenRatioByMoney": "开仓手续费率",
        "OpenRatioByVolume": "开仓手续费/手",
        "CloseRatioByMoney": "平仓手续费率",
        "CloseRatioByVolume": "平仓手续费/手",
        "CloseTodayRatioByMoney": "平今手续费率",
        "CloseTodayRatioByVolume": "平今手续费/手",
        "DeliveryYear": "交割年份",
        "DeliveryMonth": "交割月份",
        "OpenDate": "上市日期",
        "ExpireDate": "最后交易日",
        "DeliveryDate": "交割日",
        "UnderlyingInstrID": "标的合约ID",
        "UnderlyingMultiple": "标的合约乘数",
        "OptionsType": "期权类型",
        "StrikePrice": "行权价",
        "InstLifePhase": "合约状态",
    }
    # 重命名列为中文
    temp_df = temp_df.rename(columns=column_mapping)
    return temp_df


# 使用示例
if __name__ == "__main__":
    option_contract_info_ctp_df = option_contract_info_ctp()
    print(option_contract_info_ctp_df)
